/* SortedMap.java
 *	Copyright 	(c) 2002-2007 Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
* [When]			 [Who]					[What]
* 25-06-2007         dennis@brains2b.nl     rewritten to implement the Map interface instead of AbstractMap
*                                           for speed
*/
package org.brains2b.collection;
import java.io.Serializable;
import java.util.AbstractCollection;
import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
/**
 * Map implementation that can be sorted defined by a given sort order
 * <p>
 * <UL>Possible sort orders are
 * <LI>KEY
 * <LI>VALUE
 * <LI>NO_SORT
 * <p> Not sorted is the default implementation.
 * <p><i>Note:</i>When using SortedMap with a Hashtable the sort order
 * first is set to <B>KEY</B>. All other MAPs are set to <B>NO_SORT</B>
 * regardless of how the Map sorting was implemented.
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.12 [25-06-2007]
 * @since TwelBox 0.30
 */
public class SortedMap implements Map,Comparator,Serializable {
    
    /** value to set sorting on KEY */
    public static final int KEY=0;
    /** value to set sorting on VALUE */
    public static final int VALUE=1;
    /** value to set NO SORTING */
    public static final int NO_SORT=-1;
    
    private ArrayList m_content=new ArrayList();
    private int m_sortOrder=KEY;
    
    private transient Collection m_values=null;
    private transient Set m_keys=null;

    /**
     * Constructor for SortedMap.
     * <p> initializes an empty SortedMap with the supplied sort Order
     * @param sortOrder
     */
    public SortedMap(int sortOrder) {
        super();
        m_sortOrder=sortOrder;
    }
    
	/**
     * Constructor for SortedMap.
     * <p> initializes an empty SortedMap with NO_SORT
	 */
    public SortedMap() {
        this(NO_SORT);
    }
    
	/**
	 * Constructor for SortedMap.
	 * <p>Initializes the SortedMap with a Map and a given sortOrder
	 * @param map
	 * @param sortOrder
	 */
    public SortedMap(Map map, int sortOrder) {
        this(sortOrder);
        putAll(map);
    }
    
	/**
	 * Constructor for SortedMap.
	 * <p> initializes the SortedMap with a Map and no sort order
	 * @param map java.util.Map
	 */
    public SortedMap(Map map) {
        this(map,NO_SORT);
    }
    
	/**
	 * Constructor for SortedMap.
	 * <p> initializes the SortedMap with a Hashtable, sorted on keys
	 * @param ht Hashtable
	 */
    public SortedMap(Hashtable ht) {
        this(ht,KEY);
    }
    
    /** gets the sort order.
     * @return int, the sort order used by this SortedMap, with a possible
     * of three values: KEY, VALUE, NO_SORT
     */
    public int getSortOrder() {
        return m_sortOrder;
    }
    
    /** sets the sort order for this SortedMap
     * @param sortOrder int, the new sort order for this SortedMap,
     * possible values are: KEY,VALUE, NO_SORT
     */
    public void setSortOrder(int sortOrder) {
        if (sortOrder!=KEY && sortOrder!=VALUE && sortOrder!=NO_SORT) {
            throw new IllegalArgumentException("sortOrder must be either KEY|VALUE|NO_SORT");
        }
        m_sortOrder = sortOrder;
    }
    
	/** compare to Objects and returns the natural order
	 * <p>Implementation takes the given sort order into account and will
	 * compare the keys or values respectively. It will always return 1 if sort
	 * order is set to NO_SORT
	 * @see java.util.Comparator#compare(Object, Object)
	 */
    public int compare(java.lang.Object oLeft, java.lang.Object oRight) {
        if (m_sortOrder==NO_SORT) {
            return 1;
        }
        int result=-1;
        if (oLeft instanceof Entry && oRight instanceof Entry) {
            String left;
            String right;
            if (m_sortOrder==KEY) {
                left= ((Entry) oLeft).getKey().toString();
                right= ((Entry) oRight).getKey().toString();
            } else {
                left= ((Entry) oLeft).getValue().toString();
                right= ((Entry) oRight).getValue().toString();
                if (left.compareTo(right)==0) { // If they are equal sort them on key
                    left= ((Entry) oLeft).getKey().toString();
                    right= ((Entry) oRight).getKey().toString();
                }
            }
            result= left.compareTo(right);
            
        } if (!(oLeft instanceof Entry) || oLeft == null) {
            result=1;
        }
        return result;
    }
    
    
	/**
	 * @see java.util.Map#entrySet()
	 */
    public java.util.Set entrySet() {
        TreeSet ts=new TreeSet(this);
        ts.addAll(m_content);
        return ts;
    }
    
    /**
     * Overwritten the default get for performance reasons
     * <p>We bypass the entrySet() while it will sort the collection
     * first. Something we don't need to get an individual value 
	 * @see java.util.AbstractMap#get(java.lang.Object)
	 */
	public Object get(Object key) {
		Object result=null;
		if (key==null) return null;
		Iterator it=m_content.iterator();
		while(it.hasNext()) {
			MapEntry e = (MapEntry) it.next();
			if (key.equals(e.getKey())) {
				result=e.getValue();
			}
		}
		return result;
	}

	/**
     * SortedMap does not allow null for keys or values 
     * @see java.util.Map#put(Object, Object)
	 */
    public Object put(Object key, Object value) {
        if (value==null || key==null) return null;
        Object old=null;
        MapEntry se=new MapEntry(key,value);
        int idx=m_content.indexOf(se);
        if (idx==-1) {
            m_content.add(se);
        } else {
            old=((MapEntry) m_content.get(idx)).getValue();
            ((MapEntry) m_content.get(idx)).setValue(value);
        }
        return old;
    }
    
	/**
	 * @see java.util.Map#putAll(Map)
	 */
    public void putAll(Map m) {
        Iterator it=m.keySet().iterator();
        while (it.hasNext()) {
            Object o=it.next();
            put(o,m.get(o));
        }
    }
    
	/**
	 * @see java.util.Map#remove(Object)
	 */
    public Object remove(Object key) {
        MapEntry se=new MapEntry(key,key);
        Object value=null;
        // This works for finding because equals only compares on key
        int idx=m_content.indexOf(se);
        if (idx!=-1) {
            value=((MapEntry) m_content.get(idx)).getValue();
            m_content.remove(se);
        }
        return value;
    }

    public void clear() {
        m_content.clear();
        
    }

    public boolean containsKey(Object key) {
        return (get(key)!=null);
    }

    public boolean containsValue(Object value) {
        if (value==null) return false;
        for(Iterator it=m_content.iterator();it.hasNext();) {
            MapEntry e = (MapEntry) it.next();
            if (value.equals(e.getValue())) {
                return true;
            }
        }
        return false;
    }

    public boolean isEmpty() {
        return m_content.isEmpty();
    }

    public Set keySet() {
        if (m_keys == null) {
            m_keys = new AbstractSet() {
                public Iterator iterator() {
                    return new Iterator() {
                    private Iterator i = entrySet().iterator();
            
                    public boolean hasNext() {
                        return i.hasNext();
                    }
            
                    public Object next() {
                        return ((Entry)i.next()).getKey();
                    }
            
                    public void remove() {
                        i.remove();
                    }
                  };
                }
                public int size() {
                    return SortedMap.this.size();
                }
    
                public boolean contains(Object k) {
                    return SortedMap.this.containsKey(k);
                }
            };
        } 
        return m_keys;
        
    }

    public int size() {
        return m_content.size();
    }

    public Collection values() {
        if (m_values==null) {
            m_values= new AbstractCollection() {
                public Iterator iterator() {
                    return new Iterator() {
                        private Iterator i = entrySet().iterator();
            
                        public boolean hasNext() {
                            return i.hasNext();
                        }
            
                        public Object next() {
                            return ((Entry)i.next()).getValue();
                        }
            
                        public void remove() {
                            i.remove();
                        }
                    };
                }
                
                public int size() {
                    return m_content.size();
                }
                
                public boolean contains(Object o) {
                    return SortedMap.this.containsValue(o);
                }
                
            };
        }
        return m_values;
    }
        
}
