/* 	Logger.java
 * 
 * Created on: 12-10-2002
 * Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002,2003, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]			[Who]				[What]
 * 26-06-2007       dennis@brains2b.nl  Reimplemented Logger to use java.util.logging.Logger 
 *                                      and be compatible with old style Logger
 * 
 */
package org.brains2b.log;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Properties;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Filter;
import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.StreamHandler;

/** Prints log messages to selected output or if not set to default output on the
 *  basis of a defined outputlevel and Logger level
 *  <UL>Logger writes output to a defined output on the basis of the following rules
 * <LI>All output send to println(String) is always written to System.out or if set to an defined output
 * <LI>All output send to println(String,level) are written to the output if the Logger-level is equal or lower than
 * the level given.
 * <LI>All output with a level set to Error and the printException functions are written to System.err
 * or a defined output stream.
 * </UL>
 * <UL> The following levels are recognized in order in which they will get printed
 * <LI>PANIC
 * <LI>INFO
 * <LI>ERROR
 * <LI>WARNING
 * <LI>TRACE, wil also print the stack traces for any possible errors
 * <LI>DEBUG
 * <LI>DEBUG_VERBOSE, same as DEBUG, but for DEBUG statements that might
 * 	   influence performance and should not be run to often.
 * <UL>
 * <p>The default level of the Logger is ERROR, so all panic,info and error messages
 * are written.
 * <BR>
 * <p>You can influence the behaviour of Logger through properties. Those properties
 * can either be defined by setting System properties or by placing a file named
 * logger.properties in the root for the java application or in a directory conf/ in that root.
 * <p>A property can be defined on either of these levels. If they are defined on both
 * levels the System properties will overwrite the file properties.
 * <UL>The following properties are recognized:
 * <LI><b>logger.level</b> The level for the Logger 
 * <LI><b>logger.err.out</b> A filename to which to write the error output
 * <LI><b>logger.std.out</b> A filename to which to write the standard output 
 *	</UL>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.20 [26-06-2007]
 */
public class Logger extends java.util.logging.Logger {

    /**
     * Implementation of Formatter to be consistant with
     * The old-style Logger's formatting
     */
    public static class ClassicFormatter extends Formatter {

        public String format(LogRecord record) {
            String format = "";
            if (record.getLevel()==Level.SEVERE) {
                format="[  error  ] ";
            } else if (record.getLevel()==Level.WARNING) {
                format="[ warning ] ";
            }
            if (record.getThrown()==null) {
                return format + record.getMessage()+"\n";
            } else {
                StringWriter sw = new StringWriter();
                PrintWriter  pw = new PrintWriter(sw);
                pw.println(format + record.getMessage());
                pw.println("********************* [Stack Trace] ******************");
                record.getThrown().printStackTrace(pw);
                pw.println("********************* [End Trace] ******************");
                pw.flush();
                return sw.toString();
            }
        }
        
    }
    
    
	public final static int PANIC=0x0;
	public final static int INFO=0x1;
	public final static int ERROR=0x02;
	public final static int WARNING=0x04;
	public final static int TRACE=0x08;
	public final static int DEBUG=0x10;
   
    /**
	 * Output level for DEBUG statements which might have a notable influence
	 * on the overal performance.
	 * @since TwelBox 0.31
	 */
	public final static int DEBUG_VERBOSE=0x20;

    private static Logger m_instance;
    
    protected Logger(String name, String resourceBundle) {
        super(name, resourceBundle);
        setLevel(Level.INFO);
        findFile();
        findSystemProperties();
    }
	/**
	 * Creates an instance of Logger if non existed and stores the instance
	 * into a static member. 
	 * <p>just a Singleton implementation
     * @return Logger
     * @deprecated use getLogger() instead
	 */
	protected static Logger instance() {
		if (m_instance==null) {
            m_instance = new Logger("twelbox",null);   
        }
        return m_instance;
	}
    
    /** get the default TwelBox logger */
    public static Logger getLogger() {
        return instance();
    }
	
	
	/**
	 * sets the Logger level programmatically.
	 * Using this function frequently might not highten the predicatibility of
	 * the output. 
	 * <p>It was implemented for test purposes only
	 * @param level int, the print level.
	 */
	public static void setLevel(int level) {
		instance().setLevel(instance().getLevel(level));
	}
	
	/**
	 * Prints the String to the standard or defined output,
	 * irrespective of the level
	 * @param str String
	 */
	public static void println(String str) {
		instance().log(Level.INFO, str);
	}

	/**
	 * Prints the String to the standard or defined output,
	 * if the level given is smaller or equal to the Logger.level
	 * @param str String
	 * @param level int
	 */
	public static void println(String str, int level) {
        Level l = instance().getLevel(level);
        instance().log(l,str);

	}

	/**
	 * Writes the message to the standard or defined error stream if the level of
	 * logger is greater or equal to ERROR, writes the complete stack trace underneath
	 * if the logger is set to DEBUG or DEBUG_VERBOSE.
	 * @param message String
	 * @param ex Exception
	 */
	public static void printException(String message,Exception ex) {
		if (instance().isLoggable(Level.CONFIG)) {
            instance().log(Level.SEVERE, message, ex);
        } else {
            instance().log(Level.SEVERE,message);
        }
		
	}

	/**
	 * Uses printException(String,Exception) to write the Exception message to the standard or defined error stream if the level of
	 * logger is greater or equal to ERROR, and writing the complete stack trace underneath
	 * if the logger is set to DEBUG or DEBUG_VERBOSE.
	 * @param ex Exception
	 * @see #printException(String, Exception)
	 */
	public static void printException(Exception ex) {
		printException(ex.getMessage(),ex);
	}

	private void init(Properties prop) {
		String value=prop.getProperty("logger.level");
		int level=INFO;
        if (value!=null) {
            if ("TRACE".equalsIgnoreCase(value)) level=TRACE;
			if ("DEBUG".equalsIgnoreCase(value)) level=DEBUG;
			if ("DEBUG_VERBOSE".equalsIgnoreCase(value)) level=DEBUG_VERBOSE;
			if ("WARNING".equalsIgnoreCase(value)) level=WARNING;
			if ("ERROR".equalsIgnoreCase(value)) level=ERROR;
		}
        setLevel(getLevel(level));
		value=prop.getProperty("logger.err.out");
		if (value!=null) {
            try {
                FileHandler fh = new FileHandler(value,true);
                fh.setLevel(Level.SEVERE);
                fh.setFormatter(new ClassicFormatter());
                addHandler(fh);
            } catch (IOException iex) {
                value =null;   
            }
		}
        if (value==null) {
            ConsoleHandler sh= new ConsoleHandler();
            sh.setLevel(Level.SEVERE);
            sh.setFormatter(new ClassicFormatter());
            addHandler(sh);
        }
		value=prop.getProperty("logger.std.out");
		if (value!=null) {
			try {
                FileHandler fh = new FileHandler(value,true);
                fh.setFormatter(new ClassicFormatter());
                fh.setLevel(Level.FINEST);
                fh.setFilter(new Filter() {
                   public boolean isLoggable(LogRecord record) {
                       return record.getLevel()!=Level.SEVERE;  
                   } 
                });
                addHandler(fh);
            } catch (IOException iex) {
                value = null;
            }
		}
        if (value == null) {
            StreamHandler sh= new StreamHandler(System.out,new ClassicFormatter()) {
                public synchronized void publish(LogRecord record) {
                    super.publish(record);
                    flush();
                }
            };
            
            sh.setLevel(Level.FINEST);
            sh.setFilter(new Filter() {
               public boolean isLoggable(LogRecord record) {
                   return record.getLevel()!=Level.SEVERE;  
               } 
            });
            addHandler(sh);
        }
		
	}
	
	private void findFile() {
		try {
		File fl=new File("logger.properties");
		if (!fl.exists()) {
			fl=new File("conf/logger.properties");
		}
		if (fl.exists()) {
			Properties prop=new Properties();
			prop.load(new FileInputStream(fl));
			init(prop);
		}
		} catch (IOException ex) {
			// be quiet
		}
			
	}	
	
	private void findSystemProperties() {
		init (System.getProperties());
	}

    private final Level getLevel(int level) {
        switch (level) {
            case PANIC:
            case ERROR:
                return Level.SEVERE;
            case WARNING:
                return Level.WARNING;
            case TRACE:
                return Level.CONFIG;
            case DEBUG:
                return Level.FINE;
            case DEBUG_VERBOSE:
                return Level.FINEST;
            case INFO:
            default:
                return Level.INFO;
               
        }
    }
}
