/* 	DefaultTask.java
 *	Copyright (c) 2002, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]				[What]
 * 8-mrt-03		dennis@brains2b.nl	Add an exception to the Stack instead of just a message
 * 07-08-2005   dennis@brains2b.nl  Use array list to store listeners, while only one is supported
 * 12-08-2005   dennis@brains2b.nl  Don't send progress messages on finished Tickets
 */
package org.brains2b.task;

import java.util.ArrayList;
import java.util.Stack;

/** Implementing a Task as default Task
 *  Extra is the possibility to send an Event for use in
 *  progress bars and the like
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.12 [07-08-2005]
 */
public abstract class DefaultTask implements Task {
	
	private boolean m_finished=false;
	private int m_result=0;
	private Stack m_messages;
	private ArrayList m_eventList;
	
	private int m_max;
	private int m_min;
	private int m_pos;

	/**
	 * Constructor for DefaultTask.
	 */
	public DefaultTask() {
		super();
		m_eventList=new ArrayList();
	}

	/**
	 * @see Task#isFinished()
	 */
	public boolean isFinished() {
		return m_finished;
	}

	/**
	 * @see Task#setFinished(boolean)
	 */
	public void setFinished(boolean b) {
		boolean old=m_finished;
		m_finished=b;
		if(old!=b) {
			TaskEvent e=new TaskEvent(this,1);
			e.setFinished(b);
			fireTaskEvent(e);
		}
		
	}

	/**
	 * @see Task#getResult()
	 */
	public int getResult() {
		return m_result;
	}

	protected void setResult(int result) {
		m_result=result;
	}
	/**
	 * @see Task#getMessages()
	 */
	public Stack getMessages() {
		return m_messages;
	}

	
    /**
     * Adds a message to the reporting Stack
     * @param s String, the message to add to the Stack
     */
	protected void addMessage(String s) {
		if (m_messages==null) {
			m_messages=new Stack();
		}
		m_messages.add(s);	
	}
	
	/**
     * Adds an exception to the reporting Stack
     * @param t Throwable, the Exception or error that needs
     * @since Framework 0.31
     * to be added to the Stack.
     */
	protected void addException(Throwable t) {
		if (m_messages==null) {
			m_messages=new Stack();
		}
		m_messages.add(t);	
	}
	

	/***
	 * Gets the maximum position.
	 * @return Returns a int
	 */
	public int getMax() {
		return m_max;
	}


	/**
	 * Gets the mininum position.
	 * @return Returns a int
	 */
	public int getMin() {
		return m_min;
	}

	
	/**
	 * Gets the position.
	 * @return Returns a int
	 */
	public int getPosition() {
		return m_pos;
	}

	
	/**
	 * Method addTaskListener.
	 * @param listener TaskListener
	 */
	public void addTaskListener(TaskListener listener) {
        m_eventList.add(listener);
	}
	
	/**
	 * Method removeTaskListener.
	 * @param listener TaskListener
	 */
	public void removeTaskListener(TaskListener listener) {
		m_eventList.remove(listener);
	}
	
	/**
	 * Progresses the position by one, until the maximum given
	 * by max.
	 */
    protected void progress() {
        progress(++m_pos);
    }
	protected void progress(int value) {
        if (!isFinished()) {
    		m_pos=value;
    		if (m_pos>m_max) {m_pos=m_max ;}
    		TaskEvent e=new TaskEvent(this,0);
    		e.setMax(m_max);
    		e.setMin(m_min);
    		e.setPosition(m_pos);
    		e.setFinished(m_finished);
    		fireTaskEvent(e);
        }
	}
	
	/**
	 * Method fireTaskEvent.
	 * @param e TaskEvent
	 */
	protected void fireTaskEvent(TaskEvent e) {
		for (int i=0;i<m_eventList.size();i++) {
			((TaskListener) m_eventList.get(i)).taskProgressed(e);
		}
	}	
		
	/**
	 * Sets the maximum position
	 * @param max The max to set
	 */
	protected void setMax(int max) {
		m_max = max;
	}

	/**
	 * Sets the minimum position
	 * @param min The min to set
	 */
	protected void setMin(int min) {
		m_min = min;
	}

}
