/* 	StringHelper.java
 *	Copyright 	(c) 2001, Dennis Groenendijk
 * 				(c) 2002-2007, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * History
 * 25-05-2001 Added singleName stripping multiplicity from (English) names
 * 05-08-2001 Added left/rightPad
 * 30-08-2002 Handle null values, fixed leftPad and rightPad for values shorter then length
 * 23-10-2002 changes javaName to objectName, created variableName, changed memberName
 * 10-11-2002 Added matchBracket methods
 * 25-11-2006 Reimplemented initCaps, made class final, fixed speed issue MatchBracket
 * 24-06-2007 breakLine should take String and not only first character of line.separator into consideration
 * 30-06-2007   dennis@brains2b.nl      fixes for javadoc
 */

package org.brains2b.util;

/** 
 * Some static functions that help with Stringmanipulation.  
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.21.2 [30-06-2007]
 */
public final class StringHelper {

    private final static char[] LEFTQUOTE =
        new char[] { '\'', '\"', '{', '[', '(' };
    private final static char[] RIGHTQUOTE =
        new char[] { '\'', '\"', '}', ']', ')' };

    /** Creates new StringHelper,
     * protected as all functions are static */
    protected StringHelper() {
        super();
    }

    /**
     * Capitalize the first Letter in a String and
     * all letters following the char.
     * @param str String, the String that needs to be InitCapped.
     * @param c char, The character after which letters should be capitalized
     * @return String, the InitCapped String, returns null if str is null
     */
    public static String initCaps(String str, char c) {
        if (str == null)
            return null;
        char[] chr = str.toCharArray();
        for (int i=0;i < chr.length;i++) {
            if (i==0 || chr[i-1]==c) {
                chr[i] = Character.toUpperCase(chr[i]);
            }
        }
        return String.valueOf(chr);
    }

    /**
     * Strips a character from a String. The String will become
     * shortend for every occurance of the char.
     * @param str String, from which the character should be
     * removed
     * @param c char, the Character that needs to be removd
     * @return String, the String minus the characters that 
     * where removed, returns null when str is null
     */
    public static String stripper(String str, char c) {
        if (str == null)
            return null;
        StringBuffer stb = new StringBuffer();
        str = str.trim();
        for (int i = 0; i < str.length(); i++) {
            if (str.charAt(i) != c) {
                stb.append(str.charAt(i));
            }
        }
        return stb.toString();
    }

    public static int getNonBaseCharacter(String value) {
        return getNonBaseCharacter(value, 0);
    }

    public static int getNonBaseCharacter(String value, int start) {
        int idx = value.length();
        for (int i = start; i < value.length(); i++) {
            if (!Character.isLetterOrDigit(value.charAt(i))) {
                idx = i;
                break;
            }
        }
        return idx;
    }
    /**
     * Returns only the base capitilazid characters [A-Z][0-9]
     * All other characters are removed from the String
     * @param str String, the value to be 'based'
     * @return String, the 'based' value
     */
    public static String baseOnly(String str) {
        if (str == null)
            return null;
        StringBuffer stb = new StringBuffer();
        str = str.toUpperCase();
        for (int i = 0; i < str.length(); i++) {
            if ((str.charAt(i) >= 'A' & str.charAt(i) <= 'Z')
                || (str.charAt(i) >= '0' & str.charAt(i) <= '9')) {
                stb.append(str.charAt(i));
            }
        }
        return stb.toString();
    }

    /**
     * Breaks a String after a maximun number of characters
     * on the first space or punctuation before that maximum.
     * Uses the default lineseparator from the running System
     * @return String, the String including linebreaks,
     * returns null when str is null
     */
    public static final String breakLine(String str, int maxLen) {
        if (str == null)
            return null;
        StringBuffer stb = new StringBuffer(str);
        int last = 0;
        String linebreak = System.getProperty("line.separator");
        int lbc=0;
        
        for (int i = 0; i < stb.length(); i++) {
            char ch = stb.charAt(i);
            if (ch == linebreak.charAt(lbc)) {
                if (lbc==linebreak.length()-1) {
                    lbc=0;
                    last = i;
                } else {
                    lbc++;
                }
            } else if (i - last > maxLen) {
                for (int j = 0; j < i - last; j++) {
                    switch (stb.charAt(i - j)) {
                        case ';' :
                        case '.' :
                        case ',' :
                        case ' ' :
                        case ':' :
                        case '?' :
                        case '!' :
                            stb.insert(i - j + 1, linebreak);
                            last = i - j + 1;

                    }

                }
            }
        }
        return stb.toString();
    }
    /**
     * Pads a String to the left with th filler String until len is reached
     * If the String is longer then the length the string will be truncated on
     * the left to not return more then lenght characters
     * @param str String,  if str is null, it will be threated as a zero length string
     * @param filler String, the String that fills the original String on the left until length.
     * @param len int, the length of the output string
     * @return String the leftpadded String
     * @exception NullPointerException thrown if filler is null
     */
    public static String leftPad(String str, String filler, int len) {
        if (str == null) {
            str = new String();
        }
        if (filler == null)
            throw new NullPointerException("filler cannot be null");
        if (str.length() >= len) {
            return str.substring(str.length() - len);
        }
        StringBuffer stb = new StringBuffer(len);
        for (int i = 0; i < (len - str.length()); i+=filler.length()) {
            stb.insert(i, filler);
        }
        stb.append(str);
        return stb.toString();
    }

    /**
     * Pads a String to the right with the filler String until len is reached
     * If the String is longer then the length the string will be truncated on
     * the right to not return more then lenght characters
     * @param str String,  if str is null, it will be threated as a zero length string
     * @param filler String, the String that fills the original String on the left until length
     * @param len int, the length of the output string
     * @return String the leftpadded String
    	 * @exception NullPointerException thrown if filler is null
     */
    public static String rightPad(String str, String filler, int len) {
        if (str == null) {
            str = new String();
        }
        if (filler == null)
            throw new NullPointerException("filler cannot be null");
        if (str.length() >= len) {
            return str.substring(0, len);
        }
        StringBuffer stb = new StringBuffer(len);
        stb.append(str);
        for (int i = 0; i < (len - str.length()); i++) {
            stb.append(filler);
        }
        return stb.toString();
    }

    /**
     * Finds the matching bracket for the first character given by left in 
     * String line.
     * @param left char, the bracket character <b> ",',{,[,( </b> one is looking for
     * @param line String, the line to find the matching bracket in
     * @return int, the position where the bracket ends in line x,
     * 	returns -1 if no at pos no bracket could be found or if the matching
     *  bracket could not be found 
     * @see #matchBracket(int, String)
     */
    public static int matchBracket(char left, String line) {
        int idx = line.indexOf(left);
        if (idx == -1)
            return -1;
        return matchBracket(idx, line);
    }

    /**
     * Finds the matching bracket for the bracket character at pos
     * in line.
     * @param pos int, The position of the matching bracket, valid
     * brackets are <b> ",',{,[,( </b>
     * @param line String, the line to find the matching bracket in
     * @return int, the position where the bracket ends in line x,
     * 	returns -1 if no at pos no bracket could be found or if the matching
     *  bracket could not be found 
     */
    public static int matchBracket(int pos, String line) {
        char begin = 0;
        char end = 0;
        int out = -1;

        if (pos == -1)
            return pos;

        for (int i = 0; i < LEFTQUOTE.length; i++) {
            if (line.charAt(pos) == LEFTQUOTE[i]) {
                begin = LEFTQUOTE[i];
                end = RIGHTQUOTE[i];
                break;
            }
        }
        if (begin == 0 || end == 0) {
            return -1;
        }

        int cnt = 0;
        for (int i = pos+1; i < line.length(); i++) {
            if (line.charAt(i) == begin) {
                if (begin == end) {
                    out = i;
                    break;
                } 
                cnt++;
                
            }
            if (line.charAt(i) == end) {
                if (cnt == 0) {
                    out = i;
                    break;
                } 
                cnt--;
            }
        }
        return out;

    }
    
    /**
     * check is a character is a quote: &quote; or &amp;
     * @param a char, the char to test
     * @return boolean true if quote, false otherwise
     */
    public static boolean isQuoteCharacter(char a) {
        if (a == '\'' || a == '\"') {
            return true;
        }
        return false;
    }
    
    /**
     * check if a String is empty, meaning a String of length zero or null
     * @param s String, to test for empty
     * @return true if String is empty, false otherwise
     */
    public static boolean isEmpty(String s) {
    		if (s==null || s.length()==0) {
    			return true;
    		} 
    		return false;
    }
}
