/* 	LoggerTest.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Jun 25, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package test.org.brains2b.log;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.StringTokenizer;
import java.util.logging.Filter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

import junit.framework.TestCase;

import org.brains2b.log.Logger;

public class LoggerTest extends TestCase {

    static class TestLogger extends Logger {
       
        public TestLogger(String name, String resourceBundle) {
            super(name,resourceBundle);
        }
        
        public static void setOutput(StringWriter out, StringWriter err) {
            
            Handler[] handlers = getLogger().getHandlers();
            for (int i=0;i<handlers.length;i++) {
                handlers[i].close();
                getLogger().removeHandler(handlers[i]);
            }
            WriterHandler wOut = new WriterHandler(out,Level.FINE);
            wOut.setFormatter(new Logger.ClassicFormatter());
            wOut.setFilter(new Filter() {
                   public boolean isLoggable(LogRecord record) {
                       return record.getLevel()!=Level.SEVERE;  
                   } 
                });
            getLogger().addHandler(wOut);
            WriterHandler wErr = new WriterHandler(err,Level.SEVERE);
            wErr.setFormatter(new Logger.ClassicFormatter());
            getLogger().addHandler(wErr);
            
        }
     
    }
    
    static class WriterHandler extends Handler {
        private Writer m_writer;
        public WriterHandler(Writer w, Level l) {
            super();
            m_writer =w;
            setLevel(l);
        }
        
        public void close() throws SecurityException {
            try {
                m_writer.close();
            } catch (IOException iex) {}
        }

        public void flush() {
          try {
            m_writer.flush();
          } catch (IOException iex) {}
        }

        public void publish(LogRecord record) {
             try {
                 if (isLoggable(record)) {
                     m_writer.write(getFormatter().format(record));
                    
                 }
             } catch (IOException iex) {}
            
        }
        
        
    }
    private final static String TESTLINE = "1234567890qwertyuiopasdfghjklzxcvbnm";    
    private final static Exception TESTEXCEPTION = new RuntimeException(TESTLINE);    
    
    private StringWriter m_strOut;
    private StringWriter m_strErr;
    
    protected void setUp() throws Exception {
       m_strOut = new StringWriter();
       m_strErr = new StringWriter();
       TestLogger.setLevel(TestLogger.INFO);
    }
    
    public void testSetLevel() {
        TestLogger.setOutput(m_strOut,m_strErr);
        TestLogger.setLevel(TestLogger.DEBUG);
        TestLogger.println(TESTLINE,Logger.DEBUG);
        TestLogger.println(TESTLINE,Logger.ERROR);
        assertTrue(inBuffer(TESTLINE, 0, m_strOut));
        assertTrue(inBuffer("[  error  ] " + TESTLINE, 0, m_strErr));
        
        TestLogger.setLevel(TestLogger.WARNING);
        TestLogger.println(TESTLINE,Logger.DEBUG);
        TestLogger.println(TESTLINE,Logger.WARNING);
        TestLogger.println(TESTLINE,Logger.ERROR);
        assertTrue(!inBuffer(TESTLINE, 1, m_strOut));
        assertTrue( inBuffer("[ warning ] " + TESTLINE, 1, m_strOut));
        assertTrue( inBuffer("[  error  ] " + TESTLINE, 1, m_strErr));
        
        TestLogger.setLevel(TestLogger.ERROR);
        TestLogger.println(TESTLINE,Logger.DEBUG);
        TestLogger.println(TESTLINE,Logger.WARNING);
        TestLogger.println(TESTLINE,Logger.ERROR);
        assertTrue(!inBuffer(TESTLINE, 2, m_strOut));
        assertTrue(!inBuffer("[ warning ] " + TESTLINE, 2, m_strOut));
        assertTrue( inBuffer("[  error  ] " + TESTLINE, 2, m_strErr));
    }

    public void testPrintlnString() {
        TestLogger.setOutput(m_strOut,m_strErr);
        Logger.println(TESTLINE);
        assertTrue( inBuffer(TESTLINE, 0, m_strOut));
        assertTrue(!inBuffer(TESTLINE, 0, m_strErr));
        
    }

    public void testPrintExceptionStringException() {
        TestLogger.setOutput(m_strOut,m_strErr);
        TestLogger.setLevel(TestLogger.DEBUG);
        TestLogger.printException(TESTLINE,TESTEXCEPTION);
        assertTrue(inBuffer("[  error  ] " + TESTLINE, 0, m_strErr));
        assertTrue(inBuffer(TESTEXCEPTION, 0, m_strErr));
        
        TestLogger.setLevel(TestLogger.TRACE);
        TestLogger.printException(TESTLINE,TESTEXCEPTION);
        assertTrue(inBuffer(TESTEXCEPTION, 1, m_strErr));
        
        
        TestLogger.setLevel(TestLogger.ERROR);
        TestLogger.printException(TESTLINE,TESTEXCEPTION);
        assertTrue(!inBuffer(TESTEXCEPTION, 2, m_strErr));
        
    }

    public void testPrintExceptionException() {
        TestLogger.setOutput(m_strOut,m_strErr);
        TestLogger.setLevel(TestLogger.DEBUG);
        TestLogger.printException(TESTEXCEPTION);
        assertTrue(inBuffer("[  error  ] " + TESTLINE, 0, m_strErr));
        assertTrue(inBuffer(TESTEXCEPTION, 0, m_strErr));
        
        TestLogger.setLevel(TestLogger.TRACE);
        TestLogger.printException(TESTEXCEPTION);
        assertTrue(inBuffer(TESTEXCEPTION, 1, m_strErr));
        
        
        TestLogger.setLevel(TestLogger.ERROR);
        TestLogger.printException(TESTEXCEPTION);
        assertTrue(!inBuffer(TESTEXCEPTION, 2, m_strErr));

    }

    private boolean inBuffer(String s, int line, StringWriter sw) {
        StringTokenizer st = new StringTokenizer(sw.toString(),System.getProperty("line.separator"));
        if (!st.hasMoreTokens() || line>st.countTokens()-1) return false;
        for (int i=0;i<line;i++) {
            st.nextToken();
        }
        return s.equals(st.nextToken());
    }
    
    
    private boolean inBuffer(Exception ex, int line, StringWriter sw) {
        String buf = sw.toString();
        int cnt=0;
        int idx=0;
        while ((idx=buf.indexOf(ex.getClass().getName(),idx+1))!=-1) {
            cnt++;
        }
        return line==cnt-1; 
    }
    
}
